==> google_applovin_unity_ads-0.0.2.tar.gz: lib/native/native_ad_widget.dart <==
import 'dart:async';
import 'dart:io' show Platform;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import '../platform_views.dart';
import '../utils.dart';
import 'controller.dart';
import 'options.dart';

const _viewType = 'ads.native';

/// Creates a NativeAd and add it to the widget tree. Uses
/// a [PlatformView] to connect to the AdView in the platform
class NativeAd extends StatefulWidget {
  /// Creates a new Native Ad.
  /// `size` can NOT be null. If so, an `AssertionError` is thrown
  ///
  const NativeAd({
    Key? key,
    this.builder,
    this.controller,
    required this.size,
    this.options = const NativeAdOptions(),
    this.error,
    this.loading,
    this.delayToShow,
    this.loadTimeout = kDefaultLoadTimeout,
    this.nonPersonalizedAds = kDefaultNonPersonalizedAds,
    this.useHybridComposition,
    this.keywords = const [],
  }) : super(key: key);


  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Creating-a-native-ad#adbuilder)
  final AdBuilder? builder;

  /// The error placeholder. If an error happens, this widget will be shown
  ///
  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Creating-a-native-ad#loading-and-error-placeholders)
  final Widget? error;

  /// The loading placeholder. This widget will be shown while the ad is loading
  ///
  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Creating-a-native-ad#loading-and-error-placeholders)
  final Widget? loading;


  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Using-the-controller-and-listening-to-native-events)
  final NativeAdController? controller;

  ///
  final NativeSize size;

  /// The options this ad will follow.
  final NativeAdOptions options;

  /// The duration the platform view will wait to be shown.
  final Duration? delayToShow;

  /// The ad will stop loading after a specified time.
  final Duration loadTimeout;

  final bool nonPersonalizedAds;

  /// {@macro ads.keywords}
  final List<String> keywords;

  /// If null, defaults to `MobileAds.useHybridComposition`
  final bool? useHybridComposition;

  @override
  _NativeAdState createState() => _NativeAdState();
}

class _NativeAdState extends State<NativeAd>
    with AutomaticKeepAliveClientMixin<NativeAd> {
  late NativeAdController controller;
  NativeAdEvent state = NativeAdEvent.loading;

  NativeAdOptions get options => widget.options;
  double? height;

  StreamSubscription? _onEventSub;

  @override
  void initState() {
    super.initState();
    controller = widget.controller ??
        NativeAdController(loadTimeout: widget.loadTimeout);
    controller.attach(true);
    _onEventSub?.cancel();
    _onEventSub = controller.onEvent.listen((e) {
      final event = e.keys.first;
      final info = e.values.first;
      switch (event) {
        case NativeAdEvent.loading:
        case NativeAdEvent.loadFailed:
        case NativeAdEvent.loaded:
        if(NativeAdEvent.loaded != event) {
          if (info is int) height = info.toDouble();
        }
          setState(() => state = event);
          break;
        default:
          break;
      }
    });
    if (!controller.isLoaded) controller.load(timeout: widget.loadTimeout);
  }

  @override
  void didUpdateWidget(NativeAd oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.options.reloadWhenSizeChanges) {
      if (oldWidget.size != widget.size) {
        controller.load(timeout: widget.loadTimeout);
      }
    }
  }

  @override
  void dispose() {
    _onEventSub?.cancel();
    _onEventSub = null;
    // dispose the controller only if the controller was
    // created by the ad.
    if (widget.controller == null) {
      controller.dispose();
    } else {
      controller.attach(false);
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    // assertPlatformIsSupported();
    // assertVersionIsSupported();

    return LayoutBuilder(
      builder: (context, consts) {
        double height = widget.size.size.height;
        double width = widget.size.size.width;

        if (height == -1 && width == -1) {
          width = consts.biggest.width;
        }

        final params = <String, dynamic>{};
        final banner_size = (widget.size == NativeSize.SMALL_NATIVE) ? "S" : ((widget.size == NativeSize.MEDIUM_NATIVE) ? "M" : "F");


        params.addAll({
          'controllerId': controller.id,
          'size_height': height,
          'size_width': width,
          'banner_size': banner_size,
          'nonPersonalizedAds': widget.nonPersonalizedAds,
          'keywords': widget.keywords,
        });

        Widget w;
        if (Platform.isAndroid) {
          w = AndroidPlatformView(
            params: params,
            viewType: _viewType,
            delayToShow: widget.delayToShow,
            useHybridComposition: widget.useHybridComposition,
          );
        } else if (Platform.isIOS) {
          w = UiKitView(
            viewType: _viewType,
            creationParamsCodec: const StandardMessageCodec(),
            creationParams: params,
          );
        } else {
          return const SizedBox();
        }

        double? finalHeight;
        if (this.height != null && !this.height!.isNegative) {
          finalHeight = this.height;
        } else
          /* if (height == -1 && width == -2) */ {
          finalHeight = height;
        }

        w = SizedBox(height: finalHeight, child: w);
        // if (state == NativeAdEvent.loaded) {
        //   w = widget.builder?.call(context, w) ?? w;
        // }

        w = Stack(children: [
          w,
              () {
            if (!controller.isLoaded) {
              if (state == NativeAdEvent.loading) {
                return widget.loading ?? const SizedBox();
              }
              if (state == NativeAdEvent.loadFailed) {
                return widget.error ?? const SizedBox();
              }
            }
            return const SizedBox();
          }(),
        ]);

        return w;
      },
    );
  }

  @override
  bool get wantKeepAlive => true;
}

==> google_applovin_unity_ads-1.0.1.tar.gz: lib/native/native_ad_widget.dart <==
import 'dart:async';
import 'dart:ffi';
import 'dart:io' show Platform;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import '../platform_views.dart';
import '../utils.dart';
import 'controller.dart';
import 'options.dart';

const _viewType = 'ads.native';

/// Creates a NativeAd and add it to the widget tree. Uses
/// a [PlatformView] to connect to the AdView in the platform
class NativeAd extends StatefulWidget {
  ///
  const NativeAd({
    Key? key,
    this.builder,
    this.controller,
    required this.size,
    required this.nativeSizeType,
    this.options = const NativeAdOptions(),
    this.error,
    this.loading,
    this.delayToShow,
    this.loadTimeout = kDefaultLoadTimeout,
    this.nonPersonalizedAds = kDefaultNonPersonalizedAds,
    this.useHybridComposition,
    this.keywords = const [],
  }) : super(key: key);


  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Creating-a-native-ad#adbuilder)
  final AdBuilder? builder;

  /// The error placeholder. If an error happens, this widget will be shown
  ///
  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Creating-a-native-ad#loading-and-error-placeholders)
  final Widget? error;

  /// The loading placeholder. This widget will be shown while the ad is loading
  ///
  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Creating-a-native-ad#loading-and-error-placeholders)
  final Widget? loading;


  /// For more info, visit the [documentation](https://github.com/bdlukaa/native_admob_flutter/wiki/Using-the-controller-and-listening-to-native-events)
  final NativeAdController? controller;

  ///
  final NativeSize size;

  // F For Ful Ad, M For Medium Ad, S For Small Ad
  final String nativeSizeType;

  /// The options this ad will follow.
  final NativeAdOptions options;

  /// The duration the platform view will wait to be shown.
  final Duration? delayToShow;

  /// The ad will stop loading after a specified time.
  final Duration loadTimeout;

  final bool nonPersonalizedAds;

  /// {@macro ads.keywords}
  final List<String> keywords;

  /// If null, defaults to `MobileAds.useHybridComposition`
  final bool? useHybridComposition;

  @override
  _NativeAdState createState() => _NativeAdState();
}

class _NativeAdState extends State<NativeAd>
    with AutomaticKeepAliveClientMixin<NativeAd> {
  late NativeAdController controller;
  NativeAdEvent state = NativeAdEvent.loading;

  NativeAdOptions get options => widget.options;
  double? height;

  StreamSubscription? _onEventSub;

  @override
  void initState() {
    super.initState();
    controller = widget.controller ??
        NativeAdController(loadTimeout: widget.loadTimeout);
    controller.attach(true);
    _onEventSub?.cancel();
    _onEventSub = controller.onEvent.listen((e) {
      final event = e.keys.first;
      final info = e.values.first;
      switch (event) {
        case NativeAdEvent.loading:
        case NativeAdEvent.loadFailed:
        case NativeAdEvent.loaded:
        if(NativeAdEvent.loaded != event) {
          if (info is int) height = info.toDouble();
        }
          setState(() => state = event);
          break;
        default:
          break;
      }
    });
    if (!controller.isLoaded) controller.load(timeout: widget.loadTimeout);
  }

  @override
  void didUpdateWidget(NativeAd oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.options.reloadWhenSizeChanges) {
      if (oldWidget.size != widget.size) {
        controller.load(timeout: widget.loadTimeout);
      }
    }
  }

  @override
  void dispose() {
    _onEventSub?.cancel();
    _onEventSub = null;
    // dispose the controller only if the controller was
    // created by the ad.
    if (widget.controller == null) {
      controller.dispose();
    } else {
      controller.attach(false);
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    // assertPlatformIsSupported();
    // assertVersionIsSupported();

    return LayoutBuilder(
      builder: (context, consts) {
        double height = widget.size.size.height;
        double width = widget.size.size.width;

        if (height == -1 && width == -1) {
          width = consts.biggest.width;
        }

        final params = <String, dynamic>{};


        params.addAll({
          'controllerId': controller.id,
          'size_height': height,
          'size_width': width,
          'banner_size': widget.nativeSizeType,
          'nonPersonalizedAds': widget.nonPersonalizedAds,
          'keywords': widget.keywords,
        });

        Widget w;
        if (Platform.isAndroid) {
          w = AndroidPlatformView(
            params: params,
            viewType: _viewType,
            delayToShow: widget.delayToShow,
            useHybridComposition: widget.useHybridComposition,
          );
        } else if (Platform.isIOS) {
          w = UiKitView(
            viewType: _viewType,
            creationParamsCodec: const StandardMessageCodec(),
            creationParams: params,
          );
        } else {
          return const SizedBox();
        }

        double? finalHeight;
        if (this.height != null && !this.height!.isNegative) {
          finalHeight = this.height;
        } else
          /* if (height == -1 && width == -2) */ {
          finalHeight = height;
        }

        w = SizedBox(height: finalHeight, child: w);
        // if (state == NativeAdEvent.loaded) {
        //   w = widget.builder?.call(context, w) ?? w;
        // }

        w = Stack(children: [
          w,
              () {
            if (!controller.isLoaded) {
              if (state == NativeAdEvent.loading) {
                return widget.loading ?? const SizedBox();
              }
              if (state == NativeAdEvent.loadFailed) {
                return widget.error ?? const SizedBox();
              }
            }
            return const SizedBox();
          }(),
        ]);

        return w;
      },
    );
  }

  @override
  bool get wantKeepAlive => true;
}
